# dotfiles

A collection of custom resources that manage user dotfiles.

# Chef Cookie Cutter

This cookbook bundles the [Cookie Cutter](http://github.com/oclaussen/chef-cookie_cutter)
gem, which provides a lot of the dotfile functionality. All DSL extensions
from the Cookie Cutter gem are directly available if this cookbook is included.
In fact, this cookbook is pretty much the playground, testing area and main
reason that gem exists.

# Requirements

This cookbook requires Chef >= 12.5.1.


# Resources

* [bash_dotfile](#bash_dotfile) - Creates the Bash configuration file for a user, by default `~/.bashrc`.
* [bash_it_dotfile](#bash_it_dotfile) - Installs [Bash It](https://github.com/Bash-it/bash-it).
* [conky_dotfile](#conky_dotfile) - Creates a [Conky](https://github.com/brndnmtthws/conky) configuration file, by default `~/.conkyrc`.
* [generic_dotfile](#generic_dotfile) - Creates a generic configuration file.
* [git_dotfile](#git_dotfile) - Creates a [Git](https://git-scm.com) configuration file, by default `~/.gitconfig`.
* [gnupg_dotfile](#gnupg_dotfile) - Creates a [GnuPG](https://www.gnupg.org) configuration file, by default `~/.gnupg/gpg.conf`.
* [ssh_dotfile](#ssh_dotfile) - Creates a SSH configuration file, by default `~/.ssh/config`.
* [tmux_dotfile](#tmux_dotfile) - Creates a [tmux](https://tmux.github.io) configuration file, by default `~/.tmux.conf`.
* [vim_dotfile](#vim_dotfile) - Creates a [Vim](http://www.vim.org) configuration file, by default `~/.vimrc`.
* [vim_colors_dotfile](#vim_colors_dotfile) - Creates a [Vim](http://www.vim.org) colorscheme file in '~/.vim/colors/<name>.vim'.
* [vim_pathogen_dotfile](#vim_pathogen_dotfile) - Installs [Vim Pathogen](https://github.com/tpope/vim-pathogen).
* [xmonad_dotfile](#xmonad_dotfile) - Creates a [xmonad](http://xmonad.org) configuration file, by default `~/.xmonad/xmonad.hs`.
* [xresources_dotfile](#xresources_dotfile) - Creates an X resources file, by default `~/.Xresources`.
* [xsession_dotfile](#xsession_dotfile) - Creates an X startup script file, by default `~/.xsession`.
* [zsh_dotfile](#zsh_dotfile) - Creates the [Zsh](http://www.zsh.org) configuration file for a user, by default `~/.zshrc`.
* [oh_my_zsh_dotfile](#oh_my_zsh_dotfile) - Installs [Oh My Zsh](https://github.com/robbyrussell/oh-my-zsh).

## bash_dotfile

Creates the Bash configuration file for a user, by default `~/.bashrc`.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)
- [dotfiles_shell](#dotfiles_shell)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|bash_it|resource|Include Bash It configuration. Must be a `bash_it_dotfile` resource or a block.|`nil`|
|prompt|block|Configure the Bash prompt (see [Prompt Builder](#bash-prompt)).|`nil`|
|option|name|Set either a [Bash option](https://www.gnu.org/software/bash/manual/html_node/The-Set-Builtin.html) or [Shell option](http://tldp.org/LDP/abs/html/options.html). This attribute can be specified multiple times.|`nil`|

### Bash Prompt

The `bash_dotfile` supports the Prompt Builder and adds the following options:

* `date`: The date in "Weekday Month Date" format (e.g., "Tue May 26")
* `hostname_short`: The hostname up to the first `.'
* `hostname_long`: The full hostname
* `hostname`: The full hostname (equivalent to `hostname_long`)
* `jobs`: The number of jobs currently managed by the shell
* `tty`: The basename of the shell's terminal device name
* `shell`: The name of the shell
* `time`: The current time in 24-hour HH:MM:SS format
* `time_24h`: The current time in 24-hour HH:MM:SS format
* `time_12h`: The current time in 12-hour HH:MM:SS format
* `time_am_pm`: The current time in 12-hour am/pm format
* `user`: The username of the current user
* `version`: The version of bash (e.g., 2.00)
* `version_long`: The release of bash, version + patch level (e.g., 2.00.0)
* `cwd`: The current working directory, with $HOME abbreviated with a tilde
* `cwd_base`: The basename of the current working directory, with $HOME abbreviated with a tilde
* `history`: The history number of this command
* `command`: The command number of this command
* `pound`: If the effective UID is 0, a #, otherwise a $


## bash_it_dotfile

Installs [Bash It](https://github.com/Bash-it/bash-it). This resource alone
will not configure Bash to use Bash It. To do this, add this resource to a
`bash_dotfile` resource.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|theme|name|Set the Bash It theme.|`"bobby"`|
|option|name, value|Export a Bash It option. This attribute can be specified multiple times.|`nil`|
|feature|name, type|Enables a Bash It feature. This attribute can be specified multiple times.|`nil`|
|aliases|name|Enables an 'aliases' feature. This attribute can be specified multiple times.|`nil`|
|completion|name|Enables a 'completion' feature. This attribute can be specified multiple times.|`nil`|
|plugin|name|Enables a 'plugin' feature. This attribute can be specified multiple times.|`nil`|

## conky_dotfile

Creates a [Conky](https://github.com/brndnmtthws/conky) configuration file,
by default `~/.conkyrc`.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|text|text|Set the Conky TEXT contents|`nil`|
|option|name, value|Set a Conky option. This attribute can be specified multiple times.|`nil`|

## generic_dotfile

Creates a generic configuration file.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|parameters|hash|A hash of parameters that can be used in the dotfile.|`{}`|

## git_dotfile

Creates a [Git](https://git-scm.com) configuration file, by default
`~/.gitconfig`.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|url|name, block|Define a URL option. Can be passed an additional block with options. This attribute can be specified multiple times.|`nil`|
|git_alias|name, command|Define a Git alias. This attribute can be specified multiple times.|`nil`|
|color|type, name, foreground: nil, background: nil, attribute: nil|Define a color option. This attribute can be specified multiple times.|`nil`|
|option|section, subsection=nil, name, value|Set a Git option. This attribute can be specified multiple times.|`nil`|
|include_path|path|Include an additional config file. This attribute can be specified multiple times.|`nil`|

## gnupg_dotfile

Creates a [GnuPG](https://www.gnupg.org) configuration file, by default
`~/.gnupg/gpg.conf`.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|option|name, value...|Set a GnuPG option. This attribute can be specified multiple times.|`nil`|

## ssh_dotfile

Creates a SSH configuration file, by default `~/.ssh/config`.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|option|name, value|Set a global SSH option. This attribute can be specified multiple times.|`nil`|
|host|name, block|Define a host. Can be passed an additional block with options. This attribute can be specified multiple times.|`nil`|

## tmux_dotfile

Creates a [tmux](https://tmux.github.io) configuration file, by default
`~/.tmux.conf`.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|prefix_key|key|Rebind the tmux prefix key.|`nil`|
|option|name, value=true, scope: ''|Set a tmux option with the given scope. This attribute can be specified multiple times.|`nil`|
|session_option|name, value=true|Set a tmux session option (default scope). This attribute can be specified multiple times.|`nil`|
|window_option|name, value=true|Set a tmux window option (scope = 'w'). This attribute can be specified multiple times.|`nil`|
|server_option|name, value=true|Set a tmux server option (scope = 's'). This attribute can be specified multiple times.|`nil`|
|bind_key|key, command, arguments..., no_prefix: false, key_table: nil|Bind a key to a command. This attribute can be specified multiple times.|`nil`|
|unbind_key|key|Remove an existing keybinding. This attribute can be specified multiple times.|`nil`|

## vim_dotfile

Creates a [Vim](http://www.vim.org) configuration file, by default `~/.vimrc`.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|colors|name|Set the Vim colorscheme.|`nil`|
|pathogen|resource|Include Vim Pathogen configuration. Must be a `vim_pathogen_dotfile` resource or a block.|`nil`|
|option|name, value=true|Set a Vim option. This attribute can be specified multiple times.|`nil`|
|global_variable|name, value=true|Set a global Vim variable. This attribute can be specified multiple times.|`nil`|
|directory_option|name, directory|Set a Vim option to a directory relative to home, and create that directory. This attribute can be specified multiple times.|`nil`|

## vim_colors_dotfile

Creates a [Vim](http://www.vim.org) colorscheme file in '~/.vim/colors/<name>.vim'.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|colors_name|name|Name of the colorscheme, also determines the filename.|`nil`|
|background|background|If the colorscheme should set a light or dark background.|`nil`|
|highlight|name, foreground: nil, background: nil, attribute: nil|Define a highlight option. This attribute can be specified multiple times.|`nil`|

## vim_pathogen_dotfile

Installs [Vim Pathogen](https://github.com/tpope/vim-pathogen). This resource
alone will not configure Vim to use Pathogen. To do this, add this resource to
a `vim_dotfile` resource.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|plugin_directory|directory|The target directory where Pathogen stores itself and plugins.|`".vim/bundle"`|
|install_plugin|name, git_repository|Install a plugin for Pathogen. This attribute can be specified multiple times.|`nil`|

## xmonad_dotfile

Creates a [xmonad](http://xmonad.org) configuration file, by default
`~/.xmonad/xmonad.hs`.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|import|module|Import a haskell module. This attribute can be specified multiple times.|`nil`|
|config|config|Name of the function that supplies the xmonad config object.|`"defaultConfig"`|
|config_modifier|function|Function that is applied to the config object. This attribute can be specified multiple times.|`nil`|
|terminal|name|Name of the terminal emulator that is spawned by default.|`"xterm"`|
|focus_follows_mouse|boolean|Whether the window focus should follow the mouse cursor.|`true`|
|border_width|width|Size of the border around windows.|`1`|
|mod_mask|key|Name of the modifier key. Affects all key bindings.|`"mod1Mask"`|
|workspaces|expression|Haskell expression that defines the available workspaces.|`"map show [1 .. 9 :: Int]"`|
|normal_border_color|color|Border color for normal windows.|`"#dddddd"`|
|focused_border_color|color|Border color for focused windows.|`"#ff0000"`|
|key_binding|key, action:, shift: false|Bind an action to a key. This attribute can be specified multiple times.|`nil`|
|mouse_binding|key, action: shift: false|Bind an action to a mouse button. This attribute can be specified multiple times.|`nil`|
|layout_hook|function|Name of a Haskell function that is used as the layout hook.|`nil`|
|manage_hook|function|Name of a Haskell function that is used as the manage hook.|`nil`|
|handle_event_hook|function|Name of a Haskell function that is used as the manage event hook.|`nil`|
|log_hook|function|Name of a Haskell function that is used as the log hook.|`nil`|
|startup_hook|function|Name of a Haskell function that is used as the startup hook.|`nil`|
|haskell_block|text|Define a plain Haskell block in the configuration file. This attribute can be specified multiple times.|`nil`|

## xresources_dotfile

Creates an X resources file, by default `~/.Xresources`.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|xresource|name, value|Configure an X resource. This attribute can be specified multiple times.|`nil`|

## xsession_dotfile

Creates an X startup script file, by default `~/.xsession`.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|wm_command|command|Shell command that represents the session. The session will stop when this command returns.|`nil`|
|command|command|Shell command that should be executed at the start of the session. This attribute can be specified multiple times.|`nil`|
|xresources_from_file|file|Merge the configuration from the given files into xrdb. This attribute can be specified multiple times.|`nil`|
|xresource|name, value|Merge a given X resource configuration directly into xrdb. This attribute can be specified multiple times.|`nil`|
|option|command, option, argument...|Set a X option with either the [xsetroot](http://www.x.org/archive/X11R7.5/doc/man/man1/xsetroot.1.html) or [xset](http://www.x.org/archive/X11R7.5/doc/man/man1/xset.1.html) command. This attribute can be specified multiple times.|`nil`|
|cursor|name|Set the X cursor.|`nil`|
|background_image|name|Set the X background image to a bitmap.|`nil`|
|background_color|color|Set the X background to a solid color.|`nil`|
|bell|arguments|Configure the bell. Arguments can be 'off' or a combination of `volume:`, `pitch:` and `duration:`.|`nil`|
|click|arguments|Configure key clicks. Arguments can be 'on', 'off' or a volume in percent.|`nil`|
|dpms|arguments|Configure DPMS parameters. Arguments can be 'off', or a combination of `standby:`, `suspend:` and `off:`.|`nil`|
|font_path|path|Append an element to the current font path.|`nil`|
|mouse|acceleration: 2, threshold: 0|Configure mouse parameters.|`nil`|
|pixel|entry:, color:|Configure pixel color values in the color map.|`nil`|
|screen_saver|arguments|Configure screen saver parameters. Arguments can be 'on' or 'off'.|`nil`|

## zsh_dotfile

Creates the [Zsh](http://www.zsh.org) configuration file for a user, by
default `~/.zshrc`.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)
- [dotfiles_shell](#dotfiles_shell)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|oh_my_zsh|resource|Include Oh My Zsh configuration. Must be a `oh_my_zsh_dotfile` resource or a block.|`nil`|
|prompt|block|Configure the Zsh prompt (see [Prompt Builder](#zsh-prompt)).|`nil`|
|option|name|Set a [Zsh option](http://linux.die.net/man/1/zshoptions). This attribute can be specified multiple times.|`nil`|

## oh_my_zsh_dotfile

Installs [Oh My Zsh](https://github.com/robbyrussell/oh-my-zsh). This resource
alone will not configure Zsh to use Oh My Zsh. To do this, add this resource
to a `zsh_dotfile` resource.


This resource includes the following mixins:

- [dotfiles_user_info](#dotfiles_user_info)
- [dotfiles_dotfile](#dotfiles_dotfile)

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|plugin_directory|directory|Directory that Oh My Zsh uses to store plugins, relative to home directory.|`"custom/plugins"`|
|theme|name|The selected Oh My Zsh theme.|`"robbyrussell"`|
|option|name, value|Export a Oh My Zsh option. This attribute can be specified multiple times.|`nil`|
|plugin|name, git: nil, enable: true|Install and enable a Oh My Zsh plugin. This attribute can be specified multiple times.|`nil`|

# Mixin Resources

[Mixin resources](https://github.com/oclaussen/chef-cookie-cutter#mixins) are
resources that are used like ruby mixins. This means that they can not be used
directly, but add common functionality to other resources.

* [dotfiles_dotfile](#dotfiles_dotfile) - This is the base resource for dotfiles.
* [dotfiles_shell](#dotfiles_shell) - This is the base resources for shell dotfiles.
* [dotfiles_user_info](#dotfiles_user_info) - This resource adds a set of properties to a dotfile resource.

## dotfiles_dotfile

This is the base resource for dotfiles. It declares default actions and
common properties for all dotfiles. A dotfile can have various sources
(template, file or Git repository).
In addition, a dotfile can create additional required directories, or
perform additional actions in form of Chef resources.


|Action|Description|Default|
|------|-----------|-------|
|create|Creates the dotfile|true|

|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|target|target|The file or directory created, implementing the dotfile.|`nil`|
|executable|executable|Whether the dotfile should be executable.|`false`|
|template_source|name, cookbook: 'dotfiles'|Configure a template as the dotfile source.|`nil`|
|file_source|name, cookbook: 'dotfiles'|Configure a cookbook file as the dotfile source.|`nil`|
|remote_source|name|Configure a remote file as the dotfile source.|`nil`|
|git_source|repository, reference: 'master'|Configure a Git repository as the dotfile source.|`nil`|
|create_directory|name|Creates a directory relative to the home directory. This attribute can be specified multiple times.|`nil`|
|additional_action|action|A proc defining an additional action to be performed. This attribute can be specified multiple times.|`nil`|

## dotfiles_shell

This is the base resources for shell dotfiles. It adds some common properties
for shells.


|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|shell_alias|name, command|Define a shell alias. This attribute can be specified multiple times.|`nil`|
|variable|name, value|Export an environment variable. This attribute can be specified multiple times.|`nil`|
|path|directory|Add a directory to the PATH environment variable. This attribute can be specified multiple times.|`nil`|
|source_file|file|Source an external file for custom configuration. This attribute can be specified multiple times.|`nil`|
|script|text|Include a snippet of plain shell script. This attribute can be specified multiple times.|`nil`|

### Prompt Builder

The dotfiles cookbook contains a prompt builder for shell dotfiles, to
simplify the definition of custom prompts. The prompt builder is extended by
the individual shell resources. By default, it includes functions to set the
foreground and background colors and style of prompt segments.

Example:

```ruby
prompt do
  segment red('foo'), blue('bar'), clear
end
```


## dotfiles_user_info

This resource adds a set of properties to a dotfile resource. It supplies
user information that can be used by the dotfile.


|Property|Arguments|Description|Default|
|--------|---------|-----------|-------|
|user|name|User name for which the dotfile is created. Defaults to the current user.||
|group|name|Group that will own the created dotfile. Defaults to GID of the provided user.||
|home|path|Home directory of the dotfile. All paths will be relative to this. Defaults to the home directory of the provided user.||
|full_name|name|Full name of the user. Can be used by all dotfiles that want to use this.||
|email|email|Email of the user. Can be used by all dotfiles that want to use it.|`nil`|

# Recipes

* [dotfiles::solarized](#dotfilessolarized) - Defines solarized palettes for convenience.
* [dotfiles::shell_options](#dotfilesshell_options) - Defines some common shell options for convenience.
* [dotfiles::tmux_options](#dotfilestmux_options) - Defines some common tmux options for convenience.

## dotfiles::solarized

Defines solarized palettes for applications in form of [shared properties]
(https://github.com/oclaussen/chef-cookie-cutter#shared-properties). The defined
blocks are:
* `:xresources_solarized_dark` the solarized dark color palette for xresources
* `:xresources_solarized_light_colors` the solarized light color palette for xresources

## dotfiles::shell_options

Defines a set common shell options in form of [shared properties]
(https://github.com/oclaussen/chef-cookie-cutter#shared-properties). The defined blocks are:
* `:shell_common_aliases`: a set of general purpose shell aliases
* `:shell_histoly`: increased history size for bash and zsh.

## dotfiles::tmux_options

Defines a set common tmux options in form of [shared properties]
(https://github.com/oclaussen/chef-cookie-cutter#shared-properties). The defined blocks are:
* `:tmux_vi_navigation`: implements vi-like navigation for tmux

# License and Maintainer

Maintainer:: Ole Claussen (<claussen.ole@gmail.com>)

Source:: https://github.com/oclaussen/chef-dotfiles

Issues:: https://github.com/oclaussen/chef-dotfiles/issues

License:: Apache 2.0
