# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# Defines a set common shell options in form of [shared properties]
# (https://github.com/oclaussen/chef-cookie-cutter#shared-properties). The defined blocks are:
# * `:shell_common_aliases`: a set of general purpose shell aliases
# * `:shell_histoly`: increased history size for bash and zsh.
#>

share_properties :shell_common_aliases do
  shell_alias 'ls', 'ls -p --color=auto'
  shell_alias 'll', 'ls -phlA --color=auto'
  shell_alias 'grep', 'grep --color=auto'
  shell_alias 'egrep', 'egrep --color=auto'
  shell_alias 'fgrep', 'fgrep --color=auto'
  shell_alias '..', 'cd ..'
  shell_alias '...', 'cd ../..'
  shell_alias '....', 'cd ../../..'
  shell_alias '.....', 'cd ../../../..'
  shell_alias 'rm', 'rm -v -I'
  shell_alias 'cp', 'cp -v'
  shell_alias 'mv', 'mv -v'
end

share_properties :shell_history do
  variable 'HISTSIZE', '10000'
  variable 'HISTFILE', ::File.join(home, '.history')

  case resource_name
  when :bash_dotfile
    variable 'HISTFILESIZE', '10000'
    variable 'HISTCONTROL', 'ignoredups'
    variable 'HISTIGNORE', '&:cd:ls:ll:pwd:exit:clear'
    option 'histappend'
  when :zsh_dotfile
    variable 'SAVEHIST', '10000'
    option 'APPEND_HISTORY'
    option 'INC_APPEND_HISTORY'
    option 'HIST_IGNORE_ALL_DUPS'
    option 'HIST_REDUCE_BLANKS'
  end
end
