# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# Defines a set common tmux options in form of [shared properties]
# (https://github.com/oclaussen/chef-cookie-cutter#shared-properties). The defined blocks are:
# * `:tmux_vi_navigation`: implements vi-like navigation for tmux
#>

share_properties :tmux_vi_navigation do
  option 'status-keys', 'vi'
  window_option 'mode-keys', 'vi'
  bind_key 'h', 'select-pane', '-L'
  bind_key 'j', 'select-pane', '-D'
  bind_key 'k', 'select-pane', '-U'
  bind_key 'l', 'select-pane', '-R'
  bind_key 'v', 'begin-selection', key_table: 'vi-copy'
  bind_key 'y', 'copy-selection', key_table: 'vi-copy'
  bind_key 'C-h', 'run' '"(tmux display-message -p \'#{pane_current_command}\' | grep -iqE \'(^|\/)g?(view|vim?)(diff)?$\' && tmux send-keys C-h) || tmux select-pane -L"', no_prefix: true
  bind_key 'C-j', 'run' '"(tmux display-message -p \'#{pane_current_command}\' | grep -iqE \'(^|\/)g?(view|vim?)(diff)?$\' && tmux send-keys C-j) || tmux select-pane -D"', no_prefix: true
  bind_key 'C-k', 'run' '"(tmux display-message -p \'#{pane_current_command}\' | grep -iqE \'(^|\/)g?(view|vim?)(diff)?$\' && tmux send-keys C-k) || tmux select-pane -U"', no_prefix: true
  bind_key 'C-l', 'run' '"(tmux display-message -p \'#{pane_current_command}\' | grep -iqE \'(^|\/)g?(view|vim?)(diff)?$\' && tmux send-keys C-l) || tmux select-pane -R"', no_prefix: true
  bind_key 'C-\'', 'run' '"(tmux display-message -p \'#{pane_current_command}\' | grep -iqE \'(^|\/)g?(view|vim?)(diff)?$\' && tmux send-keys \'C-\\\') || tmux select-pane -l"', no_prefix: true
end
