# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# rubocop:disable Style/BlockDelimiters

#<
# Installs [Bash It](https://github.com/Bash-it/bash-it). This resource alone
# will not configure Bash to use Bash It. To do this, add this resource to a
# `bash_dotfile` resource.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :bash_it_dotfile

def initialize(name, run_context = nil)
  super(name, run_context)

  target '.bash_it'
  git_source 'https://github.com/Bash-it/bash-it.git'
end

class BashItOption
  def initialize(name, value)
    @name = name
    @value = value
  end

  def to_s
    "export #{@name}=\"#{@value}\""
  end
end

class BashItFeature
  attr_reader :name
  attr_reader :type

  def initialize(name, type)
    @name = name
    @type = type
  end

  def directory
    (type == 'plugin' ? 'plugins' : type)
  end
end

#<> @property theme @args (name) Set the Bash It theme.
property :theme, kind_of: String, default: 'bobby'

#<> @property option @args (name, value) Export a Bash It option.
property :option, ::Chef::CookieCutter::FancyProperty, collect: true, coerce_class: BashItOption

#<> @property feature @args (name, type) Enables a Bash It feature.
property :feature, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: BashItFeature,
         coerce: proc { |feature|
           additional_action proc { |new_resource|
             feature_dir = ::File.join(new_resource.home, new_resource.target, feature.directory)
             script_name = "#{feature.name}.#{feature.type}.bash"
             link ::File.join(feature_dir, 'enabled', script_name) do
               to ::File.join(feature_dir, 'available', script_name)
               owner new_resource.user
               group new_resource.group
             end
           }
           feature
         }

#<> @property aliases @args (name) Enables an 'aliases' feature.
property :aliases, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce: proc { |name| feature name, 'aliases' }

#<> @property completion @args (name) Enables a 'completion' feature.
property :completion, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce: proc { |name| feature name, 'completion' }

#<> @property plugin @args (name) Enables a 'plugin' feature.
property :plugin, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce: proc { |name| feature name, 'plugin' }
