# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# Creates a [Conky](https://github.com/brndnmtthws/conky) configuration file,
# by default `~/.conkyrc`.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :conky_dotfile

provides :dotfile do |node|
  node.run_context.resource_builder.name =~ regex_from_app_name('conky')
end

def initialize(name, run_context = nil)
  super(name, run_context)

  target '.conkyrc'
  template_source 'conky/conkyrc.erb'
end

class ConkyOption
  def initialize(name, value)
    @name = name
    @value = value
  end

  def to_s
    if @value.is_a?(TrueClass)
      "#{@name} yes"
    elsif @value.is_a?(FalseClass)
      "#{@name} no"
    else
      "#{@name} #{@value}"
    end
  end
end

#<> @property text @args (text) Set the Conky TEXT contents
property :text, kind_of: String

#<> @property option @args (name, value) Set a Conky option.
property :option, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: ConkyOption
