# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# Creates a [GnuPG](https://www.gnupg.org) configuration file, by default
# `~/.gnupg/gpg.conf`.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :gnupg_dotfile

provides :dotfile do |node|
  result = false
  result ||= node.run_context.resource_builder.name =~ regex_from_app_name('gnupg')
  result ||= node.run_context.resource_builder.name =~ regex_from_app_name('gpg')
  result
end

def initialize(name, run_context = nil)
  super(name, run_context)

  target ::File.join('.gnupg', 'gpg.conf')
  template_source 'gnupg/gpg.conf.erb'
  create_directory '.gnupg'
end

class GPGOption
  def initialize(name, *args)
    @name = name
    @args = args
  end

  def to_s
    "#{@name} #{@args.join(' ')}"
  end
end

#<> @property option @args (name, value...) Set a GnuPG option.
property :option, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: GPGOption
