# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# rubocop:disable Style/BlockDelimiters

#<
# @mixin mixin/user_info
# This resource adds a set of properties to a dotfile resource. It supplies
# user information that can be used by the dotfile.
#>

#<> @property user @args (name) User name for which the dotfile is created. Defaults to the current user.
property :user,
         kind_of: String,
         default: lazy { node['current_user'] }

#<> @property group @args (name) Group that will own the created dotfile. Defaults to GID of the provided user.
property :group,
         kind_of: String,
         default: lazy {
           gid = node['etc']['passwd'][user]['gid']
           matching_groups = node['etc']['group']
           matching_groups = matching_groups.select { |_, grp| grp['gid'] == gid }
           matching_groups.keys[0]
         }

#<> @property home @args (path) Home directory of the dotfile. All paths will be relative to this. Defaults to the home directory of the provided user.
property :home,
         kind_of: String,
         default: lazy { node['etc']['passwd'][user]['dir'] }

#<> @property full_name @args (name) Full name of the user. Can be used by all dotfiles that want to use this.
property :full_name,
         kind_of: String,
         default: lazy {
           name = node['etc']['passwd'][user]['gecos']
           return user if name.nil?
           name.split(',')[0]
         }

#<> @property email @args (email) Email of the user. Can be used by all dotfiles that want to use it.
property :email,
         kind_of: String
