# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# Creates a SSH configuration file, by default `~/.ssh/config`.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :ssh_dotfile

provides :dotfile do |node|
  node.run_context.resource_builder.name =~ regex_from_app_name('ssh')
end

def initialize(name, run_context = nil)
  super(name, run_context)

  target ::File.join('.ssh', 'config')
  template_source 'ssh/config.erb'
  create_directory '.ssh'
end

class SSHOption
  def initialize(name, value)
    @name = name
    @value = value
  end

  def to_s
    printed_value = @value
    printed_value = 'yes' if printed_value.is_a?(TrueClass)
    printed_value = 'no'  if printed_value.is_a?(FalseClass)
    "#{@name} #{printed_value}"
  end
end

class SSHHost
  def initialize(name)
    @name = name
  end

  def option(name, value)
    @options ||= []
    @options << SSHOption.new(name, value)
  end

  def to_s
    result = []
    result << "Host #{@name}"
    @options.each { |o| result << "    #{o}" } unless @options.nil?
    result.join("\n")
  end
end

#<> @property option @args (name, value) Set a global SSH option.
property :option, ::Chef::CookieCutter::FancyProperty, collect: true, coerce_class: SSHOption

#<> @property host @args (name, block) Define a host. Can be passed an additional block with options.
property :host, ::Chef::CookieCutter::FancyProperty, collect: true, allow_kwargs: true, coerce_class: SSHHost
