# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# Creates a [Vim](http://www.vim.org) colorscheme file in '~/.vim/colors/<name>.vim'.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :vim_colors_dotfile

provides :dotfile do |node|
  node.run_context.resource_builder.name =~ regex_from_app_name('vim')
end

def initialize(name, run_context = nil)
  super(name, run_context)

  target lazy { ::File.join('.vim', 'colors', "#{colors_name}.vim") }
  template_source 'vim/colors.vim.erb'
end

class VimHighlight
  def initialize(name, foreground: nil, background: nil, attribute: nil)
    @name = name
    unless foreground.nil?
      @foreground = ::Chef::Dotfiles::Colors.from_property foreground
    end
    unless background.nil?
      @background = ::Chef::Dotfiles::Colors.from_property background
    end
    @attribute = attribute unless attribute.nil?
  end

  %w(foreground background attribute).each do |method|
    define_method("#{method}?") do
      !instance_variable_get("@#{method}").nil?
    end
    attr_reader method
  end

  def to_s
    result = []
    result << "hi #{@name}"
    result << "guifg=#{foreground.to_rgbcolor}" if foreground?
    result << "guibg=#{background.to_rgbcolor}" if background?
    result << "gui=#{attribute}" if attribute?
    result << "ctermfg=#{foreground.to_termcolor}" if foreground?
    result << "ctermbg=#{background.to_termcolor}" if background?
    result << "cterm=#{attribute}" if attribute?
    result.join(' ')
  end
end

#<> @property colors_name @args (name) Name of the colorscheme, also determines the filename.
property :colors_name, kind_of: String, required: true

#<> @property background @args (background) If the colorscheme should set a light or dark background.
property :background, kind_of: String

#<> @property highlight @args (name, foreground: nil, background: nil, attribute: nil) Define a highlight option.
property :highlight, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: VimHighlight
