# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

#<
# Creates a [xmonad](http://xmonad.org) configuration file, by default
# `~/.xmonad/xmonad.hs`.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :xmonad_dotfile

provides :dotfile do |node|
  node.run_context.resource_builder.name =~ regex_from_app_name('xmonad')
end

def initialize(name, run_context = nil)
  super(name, run_context)

  target ::File.join('.xmonad', 'xmonad.hs')
  template_source 'xclient/xmonad.hs.erb'
  create_directory '.xmonad'
end

class XMonadImport
  def initialize(name, as: nil, qualified: false)
    @name = name
    @as = as
    @qualified = qualified
  end

  def to_s
    result = []
    result << 'import'
    result << 'qualified' if @qualified
    result << @name
    result << "as #{@as}" unless @as.nil?
    result.join(' ')
  end
end

class XMonadKeyBinding
  def initialize(key, action:, shift: false)
    @key = key
    @action = action
    @shift = shift
  end

  def to_s
    "((modm#{'.|. shiftMask' if @shift}, #{@key}), #{@action})"
  end
end

#<> @property import @args (module) Import a haskell module.
property :import, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: XMonadImport

#<> @property config @args (config) Name of the function that supplies the xmonad config object.
property :config, kind_of: String, default: 'defaultConfig'

#<> @property config_modifier @args (function) Function that is applied to the config object.
property :config_modifier, ::Chef::CookieCutter::FancyProperty,
         collect: true

#<> @property terminal @args (name) Name of the terminal emulator that is spawned by default.
property :terminal,
         kind_of: String,
         default: 'xterm',
         coerce: proc { |value| "\"#{value}\"" }

#<> @property focus_follows_mouse @args (boolean) Whether the window focus should follow the mouse cursor.
property :focus_follows_mouse,
         kind_of: String,
         default: true,
         coerce: proc { |value| value.to_s.capitalize }

#<> @property border_width @args (width) Size of the border around windows.
property :border_width,
         default: 1,
         kind_of: Integer

#<> @property mod_mask @args (key) Name of the modifier key. Affects all key bindings.
property :mod_mask,
         default: 'mod1Mask',
         kind_of: String

#<> @property workspaces @args (expression) Haskell expression that defines the available workspaces.
property :workspaces,
         default: 'map show [1 .. 9 :: Int]',
         kind_of: String

#<> @property normal_border_color @args (color) Border color for normal windows.
property :normal_border_color, ::Chef::CookieCutter::FancyProperty,
         default: '#dddddd',
         coerce: proc { |c| "\"#{::Chef::Dotfiles::Colors.from_property(c).to_rgbcolor}\"" }

#<> @property focused_border_color @args (color) Border color for focused windows.
property :focused_border_color, ::Chef::CookieCutter::FancyProperty,
         default: '#ff0000',
         coerce: proc { |c| "\"#{::Chef::Dotfiles::Colors.from_property(c).to_rgbcolor}\"" }

#<> @property key_binding @args (key, action:, shift: false) Bind an action to a key.
property :key_binding, ::Chef::CookieCutter::FancyProperty,
         collect: true,
         coerce_class: XMonadKeyBinding

#<> @property mouse_binding @args (key, action: shift: false) Bind an action to a mouse button.
property :mouse_binding, ::Chef::CookieCutter::FancyProperty,
         collect: true,
         coerce_class: XMonadKeyBinding

#<> @property layout_hook @args (function) Name of a Haskell function that is used as the layout hook.
property :layout_hook, kind_of: String

#<> @property manage_hook @args (function) Name of a Haskell function that is used as the manage hook.
property :manage_hook, kind_of: String

#<> @property handle_event_hook @args (function) Name of a Haskell function that is used as the manage event hook.
property :handle_event_hook, kind_of: String

#<> @property log_hook @args (function) Name of a Haskell function that is used as the log hook.
property :log_hook, kind_of: String

#<> @property startup_hook @args (function) Name of a Haskell function that is used as the startup hook.
property :startup_hook, kind_of: String

#<> @property haskell_block @args (text) Define a plain Haskell block in the configuration file.
property :haskell_block, ::Chef::CookieCutter::FancyProperty,
         collect: true,
         kind_of: String
