# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# rubocop:disable Style/BlockDelimiters

#<
# Installs [Oh My Zsh](https://github.com/robbyrussell/oh-my-zsh). This resource
# alone will not configure Zsh to use Oh My Zsh. To do this, add this resource
# to a `zsh_dotfile` resource.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :oh_my_zsh_dotfile

def initialize(name, run_context = nil)
  super(name, run_context)

  target '.oh-my-zsh'
  git_source 'https://github.com/robbyrussell/oh-my-zsh.git'
end

class OhMyZshOption
  def initialize(name, value)
    @name = name
    @value = value
  end

  def to_s
    "#{@name}=\"#{@value}\""
  end
end

class OhMyZshPlugin
  attr_reader :name
  attr_reader :git
  attr_reader :enable

  def initialize(name, git: nil, enable: true)
    @name = name
    @git = git
    @enable = enable
  end

  def to_s
    @name.to_s
  end
end

#<> @property plugin_directory @args (directory) Directory that Oh My Zsh uses to store plugins, relative to home directory.
property :plugin_directory, kind_of: String, default: ::File.join('custom', 'plugins')

#<> @property theme @args (name) The selected Oh My Zsh theme.
property :theme, kind_of: String, default: 'robbyrussell'

#<> @property option @args (name, value) Export a Oh My Zsh option.
property :option, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: OhMyZshOption

#<> @property plugin @args (name, git: nil, enable: true) Install and enable a Oh My Zsh plugin.
property :plugin, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: OhMyZshPlugin,
         coerce: proc { |plugin|
           additional_action proc { |new_resource|
             git ::File.join(new_resource.home, new_resource.target, new_resource.plugin_directory, plugin.name) do
               action :sync
               repository plugin.git
               user new_resource.user
               group new_resource.group
             end
           } unless plugin.git.nil?
           plugin
         }
