# encoding: UTF-8
#
# Copyright 2015, Ole Claussen <claussen.ole@gmail.com>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# rubocop:disable Style/BlockDelimiters

#<
# Installs [Vim Pathogen](https://github.com/tpope/vim-pathogen). This resource
# alone will not configure Vim to use Pathogen. To do this, add this resource to
# a `vim_dotfile` resource.
#>

include_resource 'mixin/user_info'
include_resource 'mixin/dotfile'

resource_name :vim_pathogen_dotfile

def initialize(name, run_context = nil)
  super(name, run_context)

  target lazy { ::File.join(plugin_directory, 'vim-pathogen') }
  git_source 'https://github.com/tpope/vim-pathogen'
end

class VimPathogenPlugin
  attr_reader :name
  attr_reader :git

  def initialize(name, git)
    @name = name
    @git = git
  end
end

#<> @property plugin_directory @args (directory) The target directory where Pathogen stores itself and plugins.
property :plugin_directory,
         kind_of: String,
         default: ::File.join('.vim', 'bundle'),
         coerce: proc { |dir|
           create_directory dir
           dir
         }

#<> @property install_plugin @args (name, git_repository) Install a plugin for Pathogen.
property :install_plugin, ::Chef::CookieCutter::FancyProperty,
         collect: true, coerce_class: VimPathogenPlugin,
         coerce: proc { |plugin|
           additional_action proc { |new_resource|
             git ::File.join(new_resource.home, new_resource.plugin_directory, plugin.name) do
               action :sync
               repository plugin.git
               user new_resource.user
               group new_resource.group
             end
           }
           plugin
         }
